21dd088c8e
* Update include/rocprofiler-sdk/marker/*
- Update rocprofiler_marker_api_args_t for all API functions
- Add ROCPROFILER_MARKER_API_ID_roctxGetThreadId to rocprofiler_marker_api_id_t
* Update include/rocprofiler-sdk/marker/api_args.h
- fix include
* Update lib/common/mpl.hpp
- is_pair
- is_type_complete_v
* Update include/rocprofiler-sdk/marker/*
- fix rocprofiler_marker_api_retval_t
- add roctxGetThreadId to rocprofiler_marker_api_args_t
- fix type in enum: HsaDevice -> HsaAgent
- add table_api_id.h
* Update include/rocprofiler-sdk/marker.h
- include marker/table_api_id.h
* Update include/rocprofiler-sdk/buffer_tracing.h
- Buffer marker tracer records have begin and end timestamp
* Add lib/rocprofiler-sdk/marker
- tracing implementation for marker (roctx) library
* Update include/rocprofiler-sdk/{buffer_tracing,marker/table_api_id}.h
- rocprofiler_buffer_tracing_marker_record_t -> rocprofiler_buffer_tracing_marker_api_record_t
* Update lib/rocprofiler-sdk/buffer_tracing.cpp
- support for ROCPROFILER_BUFFER_TRACING_MARKER_API
* Update lib/rocprofiler-sdk/callback_tracing.cpp
- support for ROCPROFILER_CALLBACK_TRACING_MARKER_API
* Update lib/rocprofiler-sdk/intercept_table.cpp
- template instantiation for notify_runtime_api_registration
* Update lib/rocprofiler-sdk/registration.cpp
- enable roctx in rocprofiler_set_api_table
* Update lib/rocprofiler-sdk/marker/marker.cpp
- rocprofiler_buffer_tracing_marker_record_t -> rocprofiler_buffer_tracing_marker_api_record_t
* Update lib/rocprofiler/tests for roctx testing
- add roctx.cpp
- unit tests for roctx callback and buffer tracing
- support marker API in get_{buffer,callback}_tracing_names()
* Update lib/common/logging.cpp
- logging initialized message mentions env variable
* Update lib/common/mpl.hpp
- NOLINT for misc-definitions-in-headers
* Update lib/rocprofiler-sdk/tests/CMakeLists.txt
- include LD_LIBRARY_PATH in rocprofiler-lib-tests-shared tests
* Update lib/rocprofiler-sdk/registration.cpp
- client_library_vec_t is now vector of option<client_library>
- enables resetting the client_library after finalization
- removed acquiring registration lock when invoke_client_finalizers called via atexit
- this was causing some lock-order-inversion warnings (potential deadlock)
* Update lib/rocprofiler-sdk/agent.cpp
- model name for agent supports spaces
* Update tests/common/serialization.hpp
- add serialization support for marker tracing data structures
* Update tests/apps
- Add ROCTx markers into reproducible-runtime and transpose
* Update tests/tools/json-tools.cpp
- add marker tracing support
- remove strdup (no longer necessary)
* Update tests/kernel-tracing/validate.py
- validate marker API tracing data
* Update tests/async-copy-tracing/validate.py
- validate marker API tracing data
* Update cmake for load path resolution during testing
* Update tests/async-copy-tracing/CMakeLists.txt
- fix test LD_LIBRARY_PATH
* Update cmake/Templates/rocprofiler-sdk-roctx/config.cmake.in
- fix constructing rocprofiler-sdk-roctx::rocprofiler-sdk-roctx
191 строка
6.8 KiB
C++
191 строка
6.8 KiB
C++
// MIT License
|
|
//
|
|
// Copyright (c) 2023 Advanced Micro Devices, Inc. All rights reserved.
|
|
//
|
|
// Permission is hereby granted, free of charge, to any person obtaining a copy
|
|
// of this software and associated documentation files (the "Software"), to deal
|
|
// in the Software without restriction, including without limitation the rights
|
|
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
|
|
// copies of the Software, and to permit persons to whom the Software is
|
|
// furnished to do so, subject to the following conditions:
|
|
//
|
|
// The above copyright notice and this permission notice shall be included in
|
|
// all copies or substantial portions of the Software.
|
|
//
|
|
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
|
|
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
|
|
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
|
|
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
|
|
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
|
|
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
|
|
// THE SOFTWARE.
|
|
|
|
#include "hip/hip_runtime.h"
|
|
#include "rocprofiler-sdk-roctx/roctx.h"
|
|
|
|
#include <unistd.h>
|
|
#include <chrono>
|
|
#include <cstdio>
|
|
#include <cstdlib>
|
|
#include <iomanip>
|
|
#include <iostream>
|
|
#include <mutex>
|
|
#include <random>
|
|
#include <stdexcept>
|
|
|
|
#if defined(USE_MPI)
|
|
# include <mpi.h>
|
|
#endif
|
|
|
|
#define HIP_API_CALL(CALL) \
|
|
{ \
|
|
hipError_t error_ = (CALL); \
|
|
if(error_ != hipSuccess) \
|
|
{ \
|
|
auto _hip_api_print_lk = auto_lock_t{print_lock}; \
|
|
fprintf(stderr, \
|
|
"%s:%d :: HIP error %i : %s\n", \
|
|
__FILE__, \
|
|
__LINE__, \
|
|
static_cast<int>(error_), \
|
|
hipGetErrorString(error_)); \
|
|
throw std::runtime_error("hip_api_call"); \
|
|
} \
|
|
}
|
|
|
|
namespace
|
|
{
|
|
using auto_lock_t = std::unique_lock<std::mutex>;
|
|
auto print_lock = std::mutex{};
|
|
double nruntime = 500.0; // ms
|
|
uint32_t nspin = 1000000;
|
|
size_t nthreads = 2;
|
|
|
|
void
|
|
check_hip_error(void);
|
|
} // namespace
|
|
|
|
__global__ void
|
|
reproducible_runtime(uint32_t nspin);
|
|
|
|
void
|
|
run(int tid, int devid);
|
|
|
|
int
|
|
main(int argc, char** argv)
|
|
{
|
|
for(int i = 1; i < argc; ++i)
|
|
{
|
|
auto _arg = std::string{argv[i]};
|
|
if(_arg == "?" || _arg == "-h" || _arg == "--help")
|
|
{
|
|
fprintf(stderr,
|
|
"usage: reproducible-runtime [KERNEL RUNTIME PER THREAD (default: %f msec)] "
|
|
"[SPIN CYCLES PER KERNEL LAUNCH (default: %u)] [NUM_THREADS (default: %zu)]\n",
|
|
nruntime,
|
|
nspin,
|
|
nthreads);
|
|
exit(EXIT_SUCCESS);
|
|
}
|
|
}
|
|
|
|
if(argc > 1) nruntime = std::stod(argv[1]);
|
|
if(argc > 2) nspin = std::stoll(argv[2]);
|
|
if(argc > 3) nthreads = std::stoll(argv[3]);
|
|
|
|
printf("[reproducible-runtime] Kernel runtime per thread: %.3f msec\n", nruntime);
|
|
printf("[reproducible-runtime] Spin time per kernel: %u cycles\n", nspin);
|
|
printf("[reproducible-runtime] Number of threads: %zu\n", nthreads);
|
|
|
|
// this is a temporary workaround in omnitrace when HIP + MPI is enabled
|
|
int ndevice = 0;
|
|
HIP_API_CALL(hipGetDeviceCount(&ndevice));
|
|
printf("[reproducible-runtime] Number of devices found: %i\n", ndevice);
|
|
auto _threads = std::vector<std::thread>{};
|
|
for(size_t i = 0; i < nthreads; ++i)
|
|
_threads.emplace_back(run, i, i % ndevice);
|
|
for(auto& itr : _threads)
|
|
itr.join();
|
|
HIP_API_CALL(hipDeviceSynchronize());
|
|
HIP_API_CALL(hipDeviceReset());
|
|
|
|
return 0;
|
|
}
|
|
|
|
__global__ void
|
|
reproducible_runtime(uint32_t nspin_v)
|
|
{
|
|
for(uint32_t i = 0; i < nspin_v / 2048; i++)
|
|
asm volatile("s_sleep 32"); // ~2048 cycles -> ~1us
|
|
uint32_t remainder = nspin_v % 2048;
|
|
for(uint32_t i = 0; i < remainder / 64; i++)
|
|
asm volatile("s_sleep 1");
|
|
}
|
|
|
|
void
|
|
run(int tid, int devid)
|
|
{
|
|
auto roctx_range_id = roctxRangeStart("run");
|
|
|
|
constexpr int min_sa = 8;
|
|
constexpr int min_avail_simd = 24;
|
|
dim3 grid(min_sa * min_avail_simd);
|
|
dim3 block(32);
|
|
double time = 0.0;
|
|
hipStream_t stream = {};
|
|
hipEvent_t start = {};
|
|
hipEvent_t stop = {};
|
|
|
|
HIP_API_CALL(hipSetDevice(devid));
|
|
HIP_API_CALL(hipStreamCreate(&stream));
|
|
HIP_API_CALL(hipEventCreate(&start));
|
|
HIP_API_CALL(hipEventCreate(&stop));
|
|
|
|
do
|
|
{
|
|
roctxMark("iteration");
|
|
uint32_t cyclesleft = 2000 * 1000 * (nruntime - static_cast<double>(time));
|
|
HIP_API_CALL(hipEventRecord(start, stream));
|
|
reproducible_runtime<<<grid, block, 0, stream>>>(std::min<uint32_t>(nspin, cyclesleft));
|
|
HIP_API_CALL(hipEventRecord(stop, stream));
|
|
check_hip_error();
|
|
HIP_API_CALL(hipEventSynchronize(stop));
|
|
float elapsed = 0.0f;
|
|
HIP_API_CALL(hipEventElapsedTime(&elapsed, start, stop));
|
|
time += static_cast<double>(elapsed);
|
|
} while(time < nruntime);
|
|
|
|
HIP_API_CALL(hipStreamSynchronize(stream));
|
|
HIP_API_CALL(hipEventDestroy(start));
|
|
HIP_API_CALL(hipEventDestroy(stop));
|
|
|
|
{
|
|
auto _msg = std::stringstream{};
|
|
_msg << '[' << getpid() << "][" << tid << "] Runtime of reproducible-runtime is "
|
|
<< std::setprecision(2) << std::fixed << time << " ms (" << std::setprecision(3)
|
|
<< (time / 1000.0f) << " sec)\n";
|
|
auto_lock_t _lk{print_lock};
|
|
std::cout << _msg.str() << std::flush;
|
|
}
|
|
|
|
HIP_API_CALL(hipStreamSynchronize(stream));
|
|
HIP_API_CALL(hipStreamDestroy(stream));
|
|
|
|
roctxRangeStop(roctx_range_id);
|
|
}
|
|
|
|
namespace
|
|
{
|
|
void
|
|
check_hip_error(void)
|
|
{
|
|
hipError_t err = hipGetLastError();
|
|
if(err != hipSuccess)
|
|
{
|
|
auto_lock_t _lk{print_lock};
|
|
std::cerr << "Error: " << hipGetErrorString(err) << std::endl;
|
|
throw std::runtime_error("hip_api_call");
|
|
}
|
|
}
|
|
} // namespace
|